// 接收方类
import 'dart:typed_data';

import 'package:bluetooth_low_energy/bluetooth_low_energy.dart';
import 'package:bluetooth_low_energy_example/utils/common.dart';
import 'package:bluetooth_low_energy_example/utils/logger.dart';

import 'packet.dart';

class Receiver {
  List<Uint8List> receivedChunks = [];
  int expectedSeqNum = 1;
  Central? central;
  GATTCharacteristic? characteristic;
  PeripheralManager? manager;

  Receiver({this.central, this.characteristic, this.manager});

  void handlePacket(Packet packet) {
    if (packet.header.type == PacketType.start) {
      int fileSize = (packet.data[0] << 24) | (packet.data[1] << 16) | (packet.data[2] << 8) | packet.data[3];
      int chunkSize = (packet.data[4] << 8) | packet.data[5];
      logger.i('收到Start包: 文件大小=$fileSize, 包大小=$chunkSize');
      expectedSeqNum = 1;
      receivedChunks = [];
      sendAck(0);
    } else if (packet.header.type == PacketType.data) {
      if (packet.header.seqNum == expectedSeqNum) {
        receivedChunks.add(packet.data);
        // logger.i('收到Data包: 序号=${packet.header.seqNum}');
        sendAck(packet.header.seqNum);
        expectedSeqNum++;
      } else {
        logger.i('收到乱序Data包: 序号=${packet.header.seqNum}，期望=$expectedSeqNum');
        sendNak(expectedSeqNum);
      }
    } else if (packet.header.type == PacketType.end) {
      logger.i('收到End包，传输完成');
      sendAck(packet.header.seqNum);
    }
  }

  void sendAck(int seqNum) {
    Packet ackPacket = packPacket(PacketType.ack, seqNum, Uint8List.fromList([seqNum >> 8, seqNum & 0xFF]));
    logger.i('接收方：发送ACK: ${ackPacket.toBytes().map((e) => toHexString(e)).join(',')}');
    if (manager != null && central != null && characteristic != null) {
      manager!.notifyCharacteristic(central!, characteristic!, value: ackPacket.toBytes());
    }
  }

  void sendNak(int seqNum) {
    Packet nakPacket = packPacket(PacketType.nak, seqNum, Uint8List.fromList([seqNum >> 8, seqNum & 0xFF]));
    logger.i('接收方：发送NAK: $seqNum');
    if (manager != null && central != null && characteristic != null) {
      manager!.notifyCharacteristic(central!, characteristic!, value: nakPacket.toBytes());
    }
  }
}
